@@ -7,7 +7,8 @@ export UmfpackLU
 import Base: (\), getproperty, show, size
 using LinearAlgebra
 using LinearAlgebra: AdjOrTrans
-import LinearAlgebra: Factorization, checksquare, det, logabsdet, lu, lu!, ldiv!
+import LinearAlgebra: Factorization, AdjointFactorization, TransposeFactorization,
+    checksquare, det, logabsdet, lu, lu!, ldiv!
 
 using SparseArrays
 using SparseArrays: getcolptr, AbstractSparseMatrixCSC
@@ -20,7 +21,6 @@ import ..increment, ..increment!, ..decrement, ..decrement!
 
 using ..LibSuiteSparse
 import ..LibSuiteSparse:
-    SuiteSparse_long,
     umfpack_dl_defaults,
     umfpack_dl_report_control,
     umfpack_dl_report_info,
@@ -47,6 +47,7 @@ import ..LibSuiteSparse:
     UMFPACK_PRL,
     UMFPACK_DENSE_ROW,
     UMFPACK_DENSE_COL,
+    UMFPACK_PIVOT_TOLERANCE,
     UMFPACK_BLOCK_SIZE,
     UMFPACK_ORDERING,
     UMFPACK_FIXQ,
@@ -81,6 +82,7 @@ import ..LibSuiteSparse:
 const JL_UMFPACK_PRL = UMFPACK_PRL + 1
 const JL_UMFPACK_DENSE_ROW = UMFPACK_DENSE_ROW + 1
 const JL_UMFPACK_DENSE_COL = UMFPACK_DENSE_COL + 1
+const JL_UMFPACK_PIVOT_TOLERANCE = UMFPACK_PIVOT_TOLERANCE + 1
 const JL_UMFPACK_BLOCK_SIZE = UMFPACK_BLOCK_SIZE + 1
 const JL_UMFPACK_ORDERING = UMFPACK_ORDERING + 1
 const JL_UMFPACK_FIXQ = UMFPACK_FIXQ + 1
@@ -140,13 +142,12 @@ macro isok(A)
     :(umferror($(esc(A))))
 end
 
-# check the size of SuiteSparse_long
-if sizeof(SuiteSparse_long) == 4
-    const UmfpackIndexTypes = (:Int32,)
-    const UMFITypes = Int32
-else
+if Sys.WORD_SIZE == 64
     const UmfpackIndexTypes = (:Int32, :Int64)
     const UMFITypes = Union{Int32, Int64}
+else
+    const UmfpackIndexTypes = (:Int32,)
+    const UMFITypes = Int32
 end
 
 const UMFVTypes = Union{Float64,ComplexF64}
@@ -256,7 +257,7 @@ workspace_W_size(F::UmfpackLU) = workspace_W_size(F, has_refinement(F))
 workspace_W_size(S::Union{UmfpackLU{<:AbstractFloat}, AbstractSparseMatrixCSC{<:AbstractFloat}}, refinement::Bool) = refinement ? 5 * size(S, 2) : size(S, 2)
 workspace_W_size(S::Union{UmfpackLU{<:Complex}, AbstractSparseMatrixCSC{<:Complex}}, refinement::Bool) = refinement ? 10 * size(S, 2) : 4 * size(S, 2)
 
-const ATLU = Union{Transpose{<:Any, <:UmfpackLU}, Adjoint{<:Any, <:UmfpackLU}}
+const ATLU = Union{TransposeFactorization{<:Any, <:UmfpackLU}, AdjointFactorization{<:Any, <:UmfpackLU}}
 has_refinement(F::ATLU) = has_refinement(F.parent)
 has_refinement(F::UmfpackLU) = has_refinement(F.control)
 has_refinement(control::AbstractVector) = control[JL_UMFPACK_IRSTEP] > 0
@@ -276,7 +277,7 @@ UmfpackWS(F::ATLU, refinement::Bool=has_refinement(F)) = UmfpackWS(F.parent, ref
 A shallow copy of UmfpackLU to use in multithreaded solve applications.
 This function duplicates the working space, control, info and lock fields.
 """
-# Not using simlar helps if the actual needed size has changed as it would need to be resized again
+# Not using similar helps if the actual needed size has changed as it would need to be resized again
 Base.copy(F::UmfpackLU{Tv, Ti}, ws=UmfpackWS(F)) where {Tv, Ti} =
     UmfpackLU(
         F.symbolic,
@@ -294,8 +295,7 @@ Base.copy(F::UmfpackLU{Tv, Ti}, ws=UmfpackWS(F)) where {Tv, Ti} =
 Base.copy(F::T, ws=UmfpackWS(F)) where {T <: ATLU} =
     T(copy(parent(F), ws))
 
-Base.adjoint(F::UmfpackLU) = Adjoint(F)
-Base.transpose(F::UmfpackLU) = Transpose(F)
+Base.transpose(F::UmfpackLU) = TransposeFactorization(F)
 
 function Base.lock(f::Function, F::UmfpackLU)
     lock(F)
@@ -336,13 +336,19 @@ When `check = false`, responsibility for checking the decomposition's
 validity (via [`issuccess`](@ref)) lies with the user.
 
 The permutation `q` can either be a permutation vector or `nothing`. If no permutation vector
-is proveded or `q` is `nothing`, UMFPACK's default is used. If the permutation is not zero based, a
-zero based copy is made.
+is provided or `q` is `nothing`, UMFPACK's default is used. If the permutation is not zero-based, a
+zero-based copy is made.
 
-The `control` vector default to the package's default configs for umfpacks but can be changed passing a
-vector of length `UMFPACK_CONTROL`. See the UMFPACK manual for possible configurations. The corresponding
-variables are named `JL_UMFPACK_` since julia uses one based indexing.
+The `control` vector defaults to the Julia SparseArrays package's default configuration for UMFPACK (NB: this is modified from the UMFPACK defaults to
+disable iterative refinement), but can be changed by passing a vector of length `UMFPACK_CONTROL`, see the UMFPACK manual for possible configurations. 
+For example to reenable iterative refinement:
 
+    umfpack_control = SparseArrays.UMFPACK.get_umfpack_control(Float64, Int64) # read Julia default configuration for a Float64 sparse matrix
+    SparseArrays.UMFPACK.show_umf_ctrl(umfpack_control) # optional - display values
+    umfpack_control[SparseArrays.UMFPACK.JL_UMFPACK_IRSTEP] = 2.0 # reenable iterative refinement (2 is UMFPACK default max iterative refinement steps)
+
+    Alu = lu(A; control = umfpack_control)
+    x = Alu \\ b   # solve Ax = b, including UMFPACK iterative refinement  
 
 The individual components of the factorization `F` can be accessed by indexing:
 
@@ -401,6 +407,11 @@ lu(A::AbstractSparseMatrixCSC; check::Bool = true) = lu(float(A); check = check)
 lu(A::AdjOrTrans{T,S}; check::Bool = true) where {T<:UMFVTypes, S<:AbstractSparseMatrixCSC{T}} =
     lu(copy(A); check)
 
+LinearAlgebra._lu(A::AbstractSparseMatrixCSC; kwargs...) =
+    lu(A; kwargs...)
+LinearAlgebra._lu(::AbstractSparseMatrixCSC, ::LinearAlgebra.PivotingStrategy; kwargs...) =
+    error("Pivoting Strategies are not supported by `SparseMatrixCSC`s")
+
 """
     lu!(F::UmfpackLU, A::AbstractSparseMatrixCSC; check=true, reuse_symbolic=true, q=nothing) -> F::UmfpackLU
 
@@ -416,7 +427,7 @@ When `check = false`, responsibility for checking the decomposition's
 validity (via [`issuccess`](@ref)) lies with the user.
 
 The permutation `q` can either be a permutation vector or `nothing`. If no permutation vector
-is proveded or `q` is `nothing`, UMFPACK's default is used. If the permutation is not zero based, a
+is provided or `q` is `nothing`, UMFPACK's default is used. If the permutation is not zero based, a
 zero based copy is made.
 
 See also [`lu`](@ref)
@@ -659,7 +670,7 @@ for itype in UmfpackIndexTypes
                 throw(ArgumentError("Wi should be larger than `size(Af, 2)`"))
             end
             if workspace_W_size(lu) > length(workspace.W)
-                throw(ArguementError("W should be larger than `workspace_W_size(Af)`"))
+                throw(ArgumentError("W should be larger than `workspace_W_size(Af)`"))
             end
             @lock lu begin
                 umfpack_numeric!(lu)
@@ -684,7 +695,7 @@ for itype in UmfpackIndexTypes
                 throw(ArgumentError("Wi should be at least larger than `size(Af, 2)`"))
             end
             if workspace_W_size(lu) > length(workspace.W)
-                throw(ArguementError("W should be larger than `workspace_W_size(Af)`"))
+                throw(ArgumentError("W should be larger than `workspace_W_size(Af)`"))
             end
             @lock lu begin
                 umfpack_numeric!(lu)
@@ -933,32 +944,30 @@ LinearAlgebra.issuccess(lu::UmfpackLU) = lu.status == UMFPACK_OK
 
 ### Solve with Factorization
 
-import LinearAlgebra.ldiv!
-
 ldiv!(lu::UmfpackLU{T}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
     ldiv!(B, lu, copy(B))
-ldiv!(translu::Transpose{T,<:UmfpackLU{T}}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
-    (lu = translu.parent; ldiv!(B, transpose(lu), copy(B)))
-ldiv!(adjlu::Adjoint{T,<:UmfpackLU{T}}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
-    (lu = adjlu.parent; ldiv!(B, adjoint(lu), copy(B)))
+ldiv!(translu::TransposeFactorization{T,<:UmfpackLU{T}}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
+    ldiv!(B, translu, copy(B))
+ldiv!(adjlu::AdjointFactorization{T,<:UmfpackLU{T}}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
+    ldiv!(B, adjlu, copy(B))
 ldiv!(lu::UmfpackLU{Float64}, B::StridedVecOrMat{<:Complex}) =
     ldiv!(B, lu, copy(B))
-ldiv!(translu::Transpose{Float64,<:UmfpackLU{Float64}}, B::StridedVecOrMat{<:Complex}) =
-    (lu = translu.parent; ldiv!(B, transpose(lu), copy(B)))
-ldiv!(adjlu::Adjoint{Float64,<:UmfpackLU{Float64}}, B::StridedVecOrMat{<:Complex}) =
-    (lu = adjlu.parent; ldiv!(B, adjoint(lu), copy(B)))
+ldiv!(translu::TransposeFactorization{Float64,<:UmfpackLU{Float64}}, B::StridedVecOrMat{<:Complex}) =
+    ldiv!(B, translu, copy(B))
+ldiv!(adjlu::AdjointFactorization{Float64,<:UmfpackLU{Float64}}, B::StridedVecOrMat{<:Complex}) =
+    ldiv!(B, adjlu, copy(B))
 
 ldiv!(X::StridedVecOrMat{T}, lu::UmfpackLU{T}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
     _Aq_ldiv_B!(X, lu, B, UMFPACK_A)
-ldiv!(X::StridedVecOrMat{T}, translu::Transpose{T,<:UmfpackLU{T}}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
+ldiv!(X::StridedVecOrMat{T}, translu::TransposeFactorization{T,<:UmfpackLU{T}}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
     (lu = translu.parent; _Aq_ldiv_B!(X, lu, B, UMFPACK_Aat))
-ldiv!(X::StridedVecOrMat{T}, adjlu::Adjoint{T,<:UmfpackLU{T}}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
+ldiv!(X::StridedVecOrMat{T}, adjlu::AdjointFactorization{T,<:UmfpackLU{T}}, B::StridedVecOrMat{T}) where {T<:UMFVTypes} =
     (lu = adjlu.parent; _Aq_ldiv_B!(X, lu, B, UMFPACK_At))
 ldiv!(X::StridedVecOrMat{Tb}, lu::UmfpackLU{Float64}, B::StridedVecOrMat{Tb}) where {Tb<:Complex} =
     _Aq_ldiv_B!(X, lu, B, UMFPACK_A)
-ldiv!(X::StridedVecOrMat{Tb}, translu::Transpose{Float64,<:UmfpackLU{Float64}}, B::StridedVecOrMat{Tb}) where {Tb<:Complex} =
+ldiv!(X::StridedVecOrMat{Tb}, translu::TransposeFactorization{Float64,<:UmfpackLU{Float64}}, B::StridedVecOrMat{Tb}) where {Tb<:Complex} =
     (lu = translu.parent; _Aq_ldiv_B!(X, lu, B, UMFPACK_Aat))
-ldiv!(X::StridedVecOrMat{Tb}, adjlu::Adjoint{Float64,<:UmfpackLU{Float64}}, B::StridedVecOrMat{Tb}) where {Tb<:Complex} =
+ldiv!(X::StridedVecOrMat{Tb}, adjlu::AdjointFactorization{Float64,<:UmfpackLU{Float64}}, B::StridedVecOrMat{Tb}) where {Tb<:Complex} =
     (lu = adjlu.parent; _Aq_ldiv_B!(X, lu, B, UMFPACK_At))
 
 function _Aq_ldiv_B!(X::StridedVecOrMat, lu::UmfpackLU, B::StridedVecOrMat, transposeoptype)
